/*******************************************************************************************
This file is part of iToday.

iToday is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

iToday is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with iContact.  If not, see <http://www.gnu.org/licenses/>.
* ------------------------------------------------------------------------------
*    GraphicFuctions.cpp - the encapsulation of GDI/Shell/Imaging API for easy use
* ------------------------------------------------------------------------------
*    Revision History
*    Date                  Author         Activity ID               Activity Headline
*    2009-10-13            Congqin				none						initial
*******************************************************************************************/

#include "stdafx.h"

#include <imaging.h>
#include <initguid.h>
#include <imgguids.h>


// definition of the function
void DrawImage(HDC hdc, RECT rc, TCHAR* pszLabel)
{
    IImagingFactory *    pImgFactory = NULL;
    IImage *             pImage		 = NULL;

    CoInitializeEx( NULL, COINIT_MULTITHREADED );

    if ( SUCCEEDED( CoCreateInstance ( CLSID_ImagingFactory,
                                       NULL,
                                       CLSCTX_INPROC_SERVER,
                                       IID_IImagingFactory,
                                       (void **) &pImgFactory
                                     )))
    {
        if (SUCCEEDED( pImgFactory->CreateImageFromFile(pszLabel,
                       &pImage)))
        {
            pImage->Draw(hdc, &rc, NULL);
            pImage->Release();
        }

        pImgFactory->Release();
    }

    CoUninitialize();
}

BOOL AlphaBlt(HDC hdcDest,	  int nXOriginDest,	int nYOriginDest,
              int nWidthDest, int nHeightDest,
              HDC hdcSrc,	  int nXOriginSrc,	int nYoriginSrc,
              int nWidthSrc,  int nHeightSrc,
              BYTE alpha)
{

    BLENDFUNCTION	bf;
    bf.BlendOp				= AC_SRC_OVER;
    bf.BlendFlags			= 0;
    bf.SourceConstantAlpha	= alpha;
    bf.AlphaFormat			= 0;

    return AlphaBlend(hdcDest,	nXOriginDest, nYOriginDest,
                      nWidthDest,   nHeightDest,
                      hdcSrc,	nXOriginSrc,  nYoriginSrc,
                      nWidthSrc,	  nHeightSrc,
                      bf
                     );

}

void DrawStringData( HDC hdc, LPCTSTR data, RECT& rc ,LOGFONT lfFont )
{
    LOGFONT lfSystemFont;

    lfSystemFont.lfHeight = lfFont.lfHeight;
    lfSystemFont.lfWidth = lfFont.lfWidth;
    lfSystemFont.lfEscapement = 0;
    lfSystemFont.lfWeight = FW_BOLD;
    lfSystemFont.lfItalic = 0;
    lfSystemFont.lfUnderline = 0;
    lfSystemFont.lfStrikeOut = 0;
    lfSystemFont.lfCharSet = CHINESEBIG5_CHARSET;
    //strcpy(lfSystemFont.lfFaceName, "");

    HFONT hCustomFont = CreateFontIndirect(&lfSystemFont);
    HFONT hOldFont = (HFONT)SelectObject(hdc, hCustomFont);

    SetBkMode(hdc, TRANSPARENT);
    SetTextColor(hdc, RGB(255, 255, 255));

    TCHAR buf[20] = {'\0'};
    //wsprintf(buf, TEXT("%s") , data);

    DrawText(hdc, data, -1, &rc, DT_SINGLELINE | DT_LEFT | DT_VCENTER);

    SelectObject(hdc, hOldFont);
    DeleteObject(hCustomFont);

}

void DrawNumber(HDC hdc, int num, RECT& rc)
{
    LOGFONT lfSystemFont;

    lfSystemFont.lfHeight = 18;
    lfSystemFont.lfWidth = 8;
    lfSystemFont.lfEscapement = 0;
    lfSystemFont.lfWeight = FW_BOLD;
    lfSystemFont.lfItalic = 0;
    lfSystemFont.lfUnderline = 0;
    lfSystemFont.lfStrikeOut = 0;
    lfSystemFont.lfCharSet = CHINESEBIG5_CHARSET;
    //strcpy(lfSystemFont.lfFaceName, "");

    HFONT hCustomFont = CreateFontIndirect(&lfSystemFont);
    HFONT hOldFont = (HFONT)SelectObject(hdc, hCustomFont);

    SetBkMode(hdc, TRANSPARENT);
    SetTextColor(hdc, RGB(255, 255, 255));

    TCHAR buf[20] = {'\0'};
    wsprintf(buf, TEXT(" %d"), num);

    DrawText(hdc, buf, -1, &rc, DT_SINGLELINE | DT_LEFT | DT_VCENTER);

    SelectObject(hdc, hOldFont);
    DeleteObject(hCustomFont);
}

BOOL DrawMemDCToCurrentDC( HDC CurrentDC, HDC PublicMemoryDC ,RECT rect)
{
    BitBlt( CurrentDC,
            rect.left,
            rect.top,
            rect.right - rect.left,
            rect.bottom - rect.top,
            PublicMemoryDC,
            rect.left,
            rect.top,
            SRCCOPY
          );
    return TRUE;
}

/*******************************************************************
*			Member functions defination of class RectEx
********************************************************************/

RectEx::RectEx()
{
    left = 0;
    top = 0;
    right = 0;
    bottom = 0;
}

RectEx::RectEx(int left, int top, int right, int bottom)
{
    this->left = left;
    this->top = top;
    this->right = right;
    this->bottom = bottom;
}

RectEx::operator RECT()
{
    RECT rc = {left, top, right, bottom};
    return rc;
}

const RectEx& RectEx::operator = (const RectEx& rcex)
{
    this->left = rcex.left;
    this->top = rcex.top;
    this->right = rcex.right;
    this->bottom = rcex.bottom;

    return rcex;
}
/////////////////////////////////////////////////////////////
PointEx::PointEx()
{
    x = 0;
    y = 0;
}

PointEx::PointEx(int cx, int cy)
{
    x = cx;
    y = cy;
}

PointEx::operator POINT()
{
    POINT pt = {x, y};
    return pt;
}

const PointEx& PointEx::operator = (const PointEx& ptex)
{
    x = ptex.x;
    y = ptex.y;

    return ptex;
}


//define the DrawImageClass'function

DrawImageClass::DrawImageClass ()
{
    CoInitializeEx( NULL, COINIT_MULTITHREADED );

    if (! SUCCEEDED( CoCreateInstance ( CLSID_ImagingFactory,
                                        NULL,
                                        CLSCTX_INPROC_SERVER,
                                        IID_IImagingFactory,
                                        (void **) &pImgFactory
                                      )))
    {
        DebugPrintString(L"There is a failure! \n" );
    }

}

BOOL DrawImageClass:: DrawImageToDC(HDC hdc, RECT rc, TCHAR* ppPhotoFile )
{

    if (SUCCEEDED( pImgFactory->CreateImageFromFile(ppPhotoFile, &pImage)))
    {
        pImage->Draw(hdc, &rc, NULL);
        pImage->Release();
    }
    return TRUE;

}

DrawImageClass:: ~DrawImageClass ()
{
    pImgFactory->Release();
    CoUninitialize();

}

