/*******************************************************************************************
This file is part of iToday.

iToday is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

iToday is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with iContact.  If not, see <http://www.gnu.org/licenses/>.
* ------------------------------------------------------------------------------
*    HomePanel.h - Class HomePanel declaration
* ------------------------------------------------------------------------------
*    Revision History
*    Date                  Author         Activity ID               Activity Headline
*    2009-10-13            Congqin				none						initial
*    2009-10-14            CubeWang				none						Modify the class method name
*    2009-10-26            CubeWang				none						add Gesture
*******************************************************************************************/

#pragma once

#include "PanelBase.h"
#include <snapi.h>
#include <regext.h>
#include <vector>
#include <phone.h>
#include "FileUtility.h"

//Home Timeʱ
#define HOME_TIME_TIMER_ID		1
#define HOME_TIME_REFRESH_MS	60000

#define  WM_MISSEDEVENT  ( WM_USER + 0 )

#define ERR_ON_FAIL(hResult) \
    if( FAILED( hResult ) ) \
	{ \
		goto _ErrorLabel;\
	}

//name of the notification
typedef enum
{
    NOTI_CALL,
    NOTI_SMS,
    NOTI_EMAIL,
    NO_NOTI
} NOTI_NAME;

#define NOTI_NUM  3
//name of the missed event
typedef enum
{
    MISSEDCALL_EVENT,
    MISSEDSMS_EVENT,
    MISSEDEMAIL_EVENT ,
    NO_EVENT
} MISSED_EVENT;


class HomePanel:virtual public PanelBase
{
private:
    struct RectMap
    {
        BOOL		m_bIsEmpty;
        RECT		m_rc;
        NOTI_NAME	m_NotiName;

        RectMap(): m_bIsEmpty( FALSE ), m_NotiName(NO_NOTI)
        {
            EMPTYRECT( m_rc);
        }
    };

    RectEx  m_rcNotiPostion[ NOTI_NUM ];
    //store the rect area of each icon
    RectMap m_NotiRectList[ NOTI_NUM ];

    //indicates which icon is pressed currently
    NOTI_NAME m_CurPressedIcon;
    //the total count of missed call
    UINT m_MissedCallCount;
    //the total count of unread sms
    UINT m_MissedSMSCount;
    //the total count of unread email
    UINT m_MissedEmailCount;

protected:

public:
    HDC				m_hMemoryDC ;
    HBITMAP			m_hMemoryBm ;
    RECT			m_hRegion;
    CTimeManager	m_cTimeManager;

    HomePanel(HWND hWnd, HINSTANCE hInstance);

    BOOL InitMemoryDC( HDC hPublicMemoryDC);
    void Draw();
    BOOL DrawTime();
    void DrawNotiIcon();

    BOOL DrawMouseDownMenu(POINT pt);

    //////////////////////////////////////////////////////////////////////////
    //functions of notification
    void InitNotification(RECT rc);
    void ProcessMouseUpMenu( POINT pt);

    void SetMissedEvent( MISSED_EVENT me, UINT nEventCnt );
    void SetInconRegion( MISSED_EVENT me );
    BOOL GetMissedEvent(void);
    void Refresh(void);
    void CheckMissedEvent(void);

    void  RegisterForPhoneNotifications();
    void  CloseAllNotifications();
    //////////////////////////////////////////////////////////////////////////

    //////////////////////////////////////////////////////////////////////////
    ///Gesture and Animation
    void SnapBackToFrame();	//Stop Animation, bring the content back into the visible area
    HRESULT
    ProcessPanEnd(		//Processes the end gesture and figures out what physics engine animation is needed
        int     nTransitionSpeed,            // @parm Start speed of animation (relevant for flick end gesture). Must be a posative value
        DWORD   nTransitionAngle             // @parm Angle for start speed (relevant for flick end gesture)
    );
    BOOL ProcessAnimationTimer();
    BOOL ProcessPan(POINTS& pt);
    BOOL RecordLastPanPoint( POINTS& pt );
    BOOL CalculateStartPointInMemDC( POINTS& pt );
    //////////////////////////////////////////////////////////////////////////

    ~HomePanel();
};
