/*******************************************************************************************
This file is part of iToday.

iToday is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

iToday is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with iContact.  If not, see <http://www.gnu.org/licenses/>.
* ------------------------------------------------------------------------------
*    PanelManager.cpp - Class PanelManager implement
* ------------------------------------------------------------------------------
*    Revision History
*    Date                  Author         Activity ID               Activity Headline
*    2009-10-13            Congqin				none						initial
*    2009-10-22            CubeWang				none		InitPanelHeadmostInitPanelLater
                                                              һϵȼ߳Thread1Panelʼ
*******************************************************************************************/

#include "stdafx.h"


PanelManager::PanelManager(HWND hWnd, HINSTANCE hInstance):m_Switcher(hWnd, hInstance),
        m_HomePanel(hWnd, hInstance),
        m_ApplicationPanel(hWnd, hInstance),
        m_ContactPanel(hWnd, hInstance),
        m_PicturePanel(hWnd, hInstance),
        m_ActivedPanel(Home),
        m_ClickRegionState(SwitcherRegion)
{
    if ( NULL != hWnd )
    {
        m_hWnd = hWnd;
        m_hInst = hInstance;
        GetClientRect( hWnd, &m_rcScreen );
        m_hRealDC = GetDC( m_hWnd );
    }
    else
    {
        DebugPrintString(L"hWnd is NULL! Init PanelManager failed!\n");
    }

    EMPTYRECT( m_rcClickedRegion );
}


DWORD WINAPI ThreadProc1( PVOID pParam )
{
    DebugPrintString(L"Thread1: Begin \n");
    PanelManager * pPM = ( PanelManager * ) pParam ;

    pPM->m_ApplicationPanel.InitMemoryDC();
    pPM->m_PicturePanel.InitMemoryDC();


    DebugPrintString(L"Thread1: End \n");


    return 0;
}


BOOL PanelManager::InitPanelLater()
{
    DWORD dwThreadID;

    HANDLE hThread1 = CreateThread(NULL, 0 , ThreadProc1,
                                   (PVOID) this, 0, &dwThreadID);
    SetThreadPriority(hThread1, THREAD_PRIORITY_IDLE);
    //CloseHandle(hThread1);
    return TRUE;
}

BOOL PanelManager::InitData()
{
    if ( InitPanelHeadmost()
            && InitPanelLater() )
    {
        return TRUE;
    }
    else
        return FALSE;
}

BOOL PanelManager::InitPanelHeadmost()
{
    DebugPrintString( L"MainThread: Into PanelManager::InitPanelHeadmost\n" );

    if (  InitPublicMemoryDC() &&
            m_Switcher.InitMemoryDC ( m_hBackGroundMemoryDC )&&
            m_HomePanel.InitMemoryDC(  m_hBackGroundMemoryDC )
       )
    {
        InitForepartCurrentDC();
        return TRUE;
    }
    else
    {
        return FALSE;
    }

}

void PanelManager::InitForepartCurrentDC()
{
    POINT	pt = { 0, 0 };
    DebugPrintString( L"%d %d %d %d\n",m_rcScreen.left ,m_rcScreen.top ,m_rcScreen.right ,m_rcScreen.bottom );
    pt.y = m_rcScreen.bottom - m_rcScreen.right/5/2;
    pt.x = m_rcScreen.right/8 * ( m_ActivedPanel*2 - 1 );
    ProcessAllPanels(m_ActivedPanel, pt, TRUE );
}

void PanelManager::Draw()
{
    DrawMemDCToCurrentDC( m_hRealDC, m_HomePanel.m_hPublicMemoryDC, m_rcScreen );
}

void PanelManager::RefreshPanel( HDC hPublicMemoryDC )
{
    BitBlt( hPublicMemoryDC,
            0,
            0,
            m_rcScreen.right,
            m_rcScreen.bottom - m_rcScreen.right / 5,
            m_hBackGroundMemoryDC,
            0,
            0,
            SRCCOPY
          );

    return ;
}


BOOL PanelManager::ProcessMouseDownEvent( POINT pt )
{
    PANEL  panel = Empty;

    if (pt.y <= m_rcScreen.bottom  && pt.y >= m_rcScreen.bottom - m_rcScreen.right / 5 )
    {
        panel = m_Switcher.GetWhichPanelClicked( pt );

        if ( panel == m_ActivedPanel || panel == Empty)
        {
            return TRUE;
        }

        ProcessAllPanels( panel, pt, FALSE);
    }
    else
    {
        m_ClickRegionState = PanelRegion ;
        DrawMouseDownMenu( pt, FALSE );
    }


    return TRUE;
}

BOOL  PanelManager::ProcessAllPanels( PANEL panel ,POINT pt ,BOOL IsInitForepart )
{
    DebugPrintString(L"MainThread: into PanelManager::ProcessAllPanels() \n");

    DrawMemDCToCurrentDC(  m_HomePanel.m_hPublicSwitchMemoryDC,
                           m_HomePanel.m_hPublicMemoryDC, m_rcScreen );

    switch ( panel )
    {
    case Contact:
        RefreshPanel( m_ContactPanel.m_hPublicMemoryDC );
        m_ContactPanel.Draw();
        m_ActivedPanel	= Contact;
        break;

    case Home:
        RefreshPanel( m_HomePanel.m_hPublicMemoryDC );
        m_HomePanel.Draw();
        m_ActivedPanel	= Home;

        break;

    case eAppPanel:
        m_ApplicationPanel.Draw();
        m_ActivedPanel	= eAppPanel;
        break;

    case Picture:
        m_PicturePanel.Draw();
        m_ActivedPanel	= Picture;
        break;

    default:
        break;

    }

    m_ClickRegionState	= SwitcherRegion ;
    DrawMouseDownMenu( pt, IsInitForepart );

    return TRUE;
}

BOOL PanelManager::DrawMouseUpMenu()
{
    RECT   rect;

    rect.left = ( m_rcScreen.right / 5 ) * ( m_ActivedPanel - 1 ) +
                ( m_rcScreen.right / 25 ) * ( m_ActivedPanel );
    rect.right = rect.left + ( m_rcScreen.right / 5 );
    rect.top =  m_rcScreen.bottom - m_rcScreen.right /5;
    rect.bottom  = m_rcScreen.bottom ;


    BitBlt( m_hRealDC, rect.left, rect.top,
            rect.right - rect.left, m_rcScreen.right / 5,
            m_hBackGroundMemoryDC, rect.left, rect.top,
            SRCCOPY
          );

    return TRUE;
}


BOOL PanelManager::DrawMouseDownMenu( POINT pt ,BOOL IsInitForepart)
{
    DebugPrintString(L"MainThread: into PanelManager::DrawMouseDownMenu() \n");
    if ( m_ClickRegionState ) // switcher region
    {
        m_Switcher.DrawMouseDownMenu(pt, m_rcScreen, IsInitForepart );
        EMPTYRECT( m_rcClickedRegion );
    }
    else
    {
        switch ( m_ActivedPanel )
        {
        case Contact:
            m_ContactPanel.DrawMouseDownMenu( pt );
            break;
        case Home:
            m_HomePanel.DrawMouseDownMenu( pt );
            break;
        case eAppPanel:
            m_ApplicationPanel.DrawMouseDownMenu( pt, m_hBackGroundMemoryDC, m_rcClickedRegion );
            break;
        case Picture:

            break;
        default:
            break;
        }
    }
    return TRUE ;
}


BOOL PanelManager::ProcessMouseUpEvent(  POINT pt )
{
    DebugPrintString(L"MainThread: into PanelManager::ProcessMouseUpEvent() \n");

    if ( m_ClickRegionState == PanelRegion )
    {
        switch ( m_ActivedPanel )
        {
        case Contact:
            m_ContactPanel.ProcessMouseUpMenu ( pt );

            break;

        case Home:
            m_HomePanel.ProcessMouseUpMenu ( pt );
            break;

        case eAppPanel:

            m_ApplicationPanel.DrawMouseUpMenu(pt, m_rcClickedRegion);
            EMPTYRECT( m_rcClickedRegion );

            break;

        case Picture:

            break;

        default:
            break;

        }
    }

    return TRUE;

}


PanelManager::~PanelManager()
{
    ReleaseDC( m_hWnd, m_hRealDC );

}

BOOL PanelManager::InitPublicMemoryDC()
{
    m_hBackGroundMemoryDC = CreateCompatibleDC( m_hRealDC );
    if ( NULL == m_hBackGroundMemoryDC )
    {
        DebugPrintString(L"PanelManager::InitPublicMemoryDC: create the m_hBackGroundMemoryDC is failure! " );
        return FALSE;
    }

    m_hBackGroundMemoryBm = CreateCompatibleBitmap( m_hRealDC, m_rcScreen.right, m_rcScreen.bottom );
    if ( NULL == m_hBackGroundMemoryBm )
    {
        DebugPrintString(L"PanelManager::InitPublicMemoryDC: create the m_hBackGroundMemoryBm is failure! " );
        return FALSE;
    }
    SelectObject( m_hBackGroundMemoryDC, m_hBackGroundMemoryBm );

    DrawPublicMemoryDC();


    return TRUE;

}

void PanelManager::DrawPublicMemoryDC()
{
    TCHAR* pBackGroud = RES_PATH("background.bmp");

    DrawImage ( m_hBackGroundMemoryDC, m_rcScreen, pBackGroud );

    return ;
}

//
//  FUNCTION: ProcessGesture(HWND, UINT, WPARAM, LPARAM)
//
//  PURPOSE:  Handles processing of WM_GESTURE command.
//
BOOL PanelManager::ProcessGesture(HWND hWnd, UINT, WPARAM wParam, LPARAM lParam)
{
    BOOL fHandled = FALSE;
    GESTUREINFO gi = {sizeof(gi)};

    // Go get the gesture - will return FALSE if the gesture engine is not present in the system.
    if (TKGetGestureInfo(reinterpret_cast<HGESTUREINFO>(lParam), &gi))
    {
        PanelBase *pPanel = GetClickedPanelPoint();
        if ( NULL == pPanel)
        {
            return FALSE;
        }

        switch (wParam)
        {
        case GID_PAN:
        {
            if (m_fAnimating)
            {
                //ǰAnimationУֹͣʾǰͼ
                pPanel->SnapBackToFrame();
                m_fAnimating = false;
            }
            if (GF_BEGIN == (gi.dwFlags & GF_BEGIN))
            {
                pPanel->RecordLastPanPoint(gi.ptsLocation);
            }
            else
            {
                pPanel->CalculateStartPointInMemDC(gi.ptsLocation);
                pPanel->RecordLastPanPoint(gi.ptsLocation);

                pPanel->ProcessPan( gi.ptsLocation );
            }
            fHandled = TRUE;
        }
        break;

        case GID_SCROLL:
        {
            //ҪSwitcherĶPanelĶǷͬʱУĿǰǲͬʱġ
            if (!m_fAnimating)
            {

                HRESULT hr = pPanel->ProcessPanEnd(
                                 (int)GID_SCROLL_VELOCITY(gi.ullArguments),
                                 (int)GID_SCROLL_ANGLE(gi.ullArguments) );
                if ( SUCCEEDED(hr) )
                {
                    m_fAnimating = true; // we are now animating
                }
            }

            fHandled = TRUE;
        }
        break;

        case GID_HOLD:
            break;

        case GID_SELECT:
            break;

        case GID_END:
        {
            //ҪSwitcherĶPanelĶǷͬʱУĿǰǲͬʱġ
            if (!m_fAnimating)
            {
                HRESULT hr = pPanel->ProcessPanEnd( 0, 0 );
                if ( SUCCEEDED(hr) )
                {
                    m_fAnimating = true; // we are now animating
                }
            }
            // GID_BEGIN/GID_END should always be send to DefGestureProc
        }
        break;
        }
    }
    else
    {
        DebugPrintString(L"The gesture engine is not present in the system.\n");
    }

    return fHandled;
}

PanelBase * PanelManager::GetClickedPanelPoint()
{
    if ( SwitcherRegion == m_ClickRegionState )
    {
        return &m_Switcher;
    }
    else if ( PanelRegion == m_ClickRegionState )
    {
        switch ( m_ActivedPanel )
        {
        case Home:
            return &m_HomePanel;
            break;
        case eAppPanel:
            return &m_ApplicationPanel;
            break;
        case Contact:
            return &m_ContactPanel;
            break;
        case Picture:
            return &m_PicturePanel;
            break;
        default:
            throw (L"m_ActivedPanel's value is out of exception!");
            break;
        }
    }
    else
    {
        throw (L"m_ClickRegionState's value is out of exception!");
    }

    return NULL;
}